@@ -153,7 +153,7 @@ def validate_cmdstan_path(path: str) -> None:
         raise ValueError(f'No CmdStan directory, path {path} does not exist.')
     if not os.path.exists(os.path.join(path, 'bin', 'stanc' + EXTENSION)):
         raise ValueError(
-            'CmdStan installataion missing binaries. '
+            f'CmdStan installataion missing binaries in {path}/bin. '
             'Re-install cmdstan by running command "install_cmdstan '
             '--overwrite", or Python code "import cmdstanpy; '
             'cmdstanpy.install_cmdstan(overwrite=True)"'
@@ -213,8 +213,9 @@ def cmdstan_version() -> Optional[Tuple[int, ...]]:
     """
     try:
         makefile = os.path.join(cmdstan_path(), 'makefile')
-    except ValueError:
+    except ValueError as e:
         get_logger().info('No CmdStan installation found.')
+        get_logger().debug("%s", e)
         return None
 
     if not os.path.exists(makefile):
@@ -299,7 +300,7 @@ def cxx_toolchain_path(
         if os.path.exists(os.path.join(toolchain_root, 'mingw64')):
             compiler_path = os.path.join(
                 toolchain_root,
-                'mingw64' if (sys.maxsize > 2**32) else 'mingw32',
+                'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
                 'bin',
             )
             if os.path.exists(compiler_path):
@@ -323,7 +324,7 @@ def cxx_toolchain_path(
         elif os.path.exists(os.path.join(toolchain_root, 'mingw_64')):
             compiler_path = os.path.join(
                 toolchain_root,
-                'mingw_64' if (sys.maxsize > 2**32) else 'mingw_32',
+                'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
                 'bin',
             )
             if os.path.exists(compiler_path):
@@ -375,7 +376,7 @@ def cxx_toolchain_path(
                 if version not in ('35', '3.5', '3'):
                     compiler_path = os.path.join(
                         toolchain_root,
-                        'mingw64' if (sys.maxsize > 2**32) else 'mingw32',
+                        'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
                         'bin',
                     )
                     if os.path.exists(compiler_path):
@@ -400,7 +401,7 @@ def cxx_toolchain_path(
                 else:
                     compiler_path = os.path.join(
                         toolchain_root,
-                        'mingw_64' if (sys.maxsize > 2**32) else 'mingw_32',
+                        'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
                         'bin',
                     )
                     if os.path.exists(compiler_path):
@@ -1361,7 +1362,7 @@ def install_cmdstan(
         logger.warning('CmdStan installation failed.\n%s', str(e))
         return False
 
-    set_cmdstan_path(args.dir)
+    set_cmdstan_path(os.path.join(args.dir, f"cmdstan-{args.version}"))
 
     return True
 
