@@ -1,16 +1,19 @@
 """install_cmdstan test"""
 
 import unittest
+from test import CustomTestCase
 
 from cmdstanpy.install_cmdstan import (
+    CmdStanInstallError,
     CmdStanRetrieveError,
     is_version_available,
     latest_version,
+    rebuild_cmdstan,
     retrieve_version,
 )
 
 
-class InstallCmdStanTest(unittest.TestCase):
+class InstallCmdStanTest(CustomTestCase):
     def test_is_version_available(self):
         # check http error for bad version
         self.assertFalse(is_version_available('2.222.222-rc222'))
@@ -34,6 +37,13 @@ class InstallCmdStanTest(unittest.TestCase):
         with self.assertRaises(ValueError):
             retrieve_version('')
 
+    def test_rebuild_bad_path(self):
+        with self.modified_environ(CMDSTAN="~/some/fake/path"):
+            with self.assertRaisesRegex(
+                CmdStanInstallError, "you sure it is installed"
+            ):
+                rebuild_cmdstan(latest_version())
+
 
 if __name__ == '__main__':
     unittest.main()
