@@ -37,8 +37,8 @@ class CmdStanMLETest(unittest.TestCase):
             os.path.join(DATAFILES_PATH, 'optimize', 'rosenbrock_mle.csv')
         ]
         mle = CmdStanMLE(runset)
-        self.assertIn('CmdStanMLE: model=rosenbrock', mle.__repr__())
-        self.assertIn('method=optimize', mle.__repr__())
+        self.assertIn('CmdStanMLE: model=rosenbrock', repr(mle))
+        self.assertIn('method=optimize', repr(mle))
         self.assertEqual(mle.column_names, ('lp__', 'x', 'y'))
         self.assertAlmostEqual(mle.optimized_params_dict['x'], 1, places=3)
         self.assertAlmostEqual(mle.optimized_params_dict['y'], 1, places=3)
@@ -48,8 +48,8 @@ class CmdStanMLETest(unittest.TestCase):
             DATAFILES_PATH, 'optimize', 'rosenbrock_mle.csv'
         )
         mle = from_csv(path=csvfiles_path)
-        self.assertIn('CmdStanMLE: model=rosenbrock', mle.__repr__())
-        self.assertIn('method=optimize', mle.__repr__())
+        self.assertIn('CmdStanMLE: model=rosenbrock', repr(mle))
+        self.assertIn('method=optimize', repr(mle))
         self.assertEqual(mle.column_names, ('lp__', 'x', 'y'))
         self.assertAlmostEqual(mle.optimized_params_dict['x'], 1, places=3)
         self.assertAlmostEqual(mle.optimized_params_dict['y'], 1, places=3)
@@ -59,8 +59,8 @@ class CmdStanMLETest(unittest.TestCase):
             DATAFILES_PATH, 'optimize', 'eight_schools_mle_iters.csv'
         )
         mle = from_csv(path=csvfiles_path)
-        self.assertIn('CmdStanMLE: model=eight_schools', mle.__repr__())
-        self.assertIn('method=optimize', mle.__repr__())
+        self.assertIn('CmdStanMLE: model=eight_schools', repr(mle))
+        self.assertIn('method=optimize', repr(mle))
         self.assertEqual(
             mle.column_names,
             (
@@ -89,8 +89,8 @@ class CmdStanMLETest(unittest.TestCase):
         stan = os.path.join(DATAFILES_PATH, 'optimize', 'rosenbrock.stan')
         model = CmdStanModel(stan_file=stan)
         mle = model.optimize(algorithm='LBFGS')
-        self.assertIn('CmdStanMLE: model=rosenbrock', mle.__repr__())
-        self.assertIn('method=optimize', mle.__repr__())
+        self.assertIn('CmdStanMLE: model=rosenbrock', repr(mle))
+        self.assertIn('method=optimize', repr(mle))
         self.assertTrue(mle.converged)
         self.assertEqual(mle.column_names, ('lp__', 'x', 'y'))
         self.assertAlmostEqual(mle.stan_variable('x'), 1, places=3)
@@ -143,8 +143,8 @@ class CmdStanMLETest(unittest.TestCase):
             )
 
     def test_eight_schools(self):
-        stan = os.path.join(DATAFILES_PATH, 'optimize', 'eight_schools.stan')
-        rdata = os.path.join(DATAFILES_PATH, 'optimize', 'eight_schools.data.R')
+        stan = os.path.join(DATAFILES_PATH, 'eight_schools.stan')
+        rdata = os.path.join(DATAFILES_PATH, 'eight_schools.data.R')
         model = CmdStanModel(stan_file=stan)
         with self.assertRaises(RuntimeError):
             model.optimize(data=rdata, algorithm='LBFGS')
@@ -152,8 +152,8 @@ class CmdStanMLETest(unittest.TestCase):
         mle = model.optimize(
             data=rdata, algorithm='LBFGS', require_converged=False
         )
-        self.assertIn('CmdStanMLE: model=eight_schools', mle.__repr__())
-        self.assertIn('method=optimize', mle.__repr__())
+        self.assertIn('CmdStanMLE: model=eight_schools', repr(mle))
+        self.assertIn('method=optimize', repr(mle))
         self.assertFalse(mle.converged)
         with LogCapture() as log:
             self.assertEqual(mle.optimized_params_pd.shape, (1, 11))
@@ -192,7 +192,7 @@ class CmdStanMLETest(unittest.TestCase):
             history_size=5,
         )
         self.assertEqual(1, len(bern_mle.metadata.stan_vars_dims))
-        self.assertTrue('theta' in bern_mle.metadata.stan_vars_dims)
+        self.assertIn('theta', bern_mle.metadata.stan_vars_dims)
         self.assertEqual(bern_mle.metadata.stan_vars_dims['theta'], ())
         theta = bern_mle.stan_variable(var='theta')
         self.assertTrue(isinstance(theta, float))
@@ -219,7 +219,7 @@ class CmdStanMLETest(unittest.TestCase):
             history_size=5,
         )
         self.assertEqual(3, len(multidim_mle.metadata.stan_vars_dims))
-        self.assertTrue('y_rep' in multidim_mle.metadata.stan_vars_dims)
+        self.assertIn('y_rep', multidim_mle.metadata.stan_vars_dims)
         self.assertEqual(
             multidim_mle.metadata.stan_vars_dims['y_rep'], (5, 4, 3)
         )
@@ -231,11 +231,11 @@ class CmdStanMLETest(unittest.TestCase):
         self.assertTrue(isinstance(var_frac_60, float))
         vars = multidim_mle.stan_variables()
         self.assertEqual(len(vars), len(multidim_mle.metadata.stan_vars_dims))
-        self.assertTrue('y_rep' in vars)
+        self.assertIn('y_rep', vars)
         self.assertEqual(vars['y_rep'].shape, (5, 4, 3))
-        self.assertTrue('beta' in vars)
+        self.assertIn('beta', vars)
         self.assertEqual(vars['beta'].shape, (2,))
-        self.assertTrue('frac_60' in vars)
+        self.assertIn('frac_60', vars)
         self.assertTrue(isinstance(vars['frac_60'], float))
 
         multidim_mle_iters = multidim_model.optimize(
@@ -256,11 +256,11 @@ class CmdStanMLETest(unittest.TestCase):
         self.assertEqual(
             len(vars_iters), len(multidim_mle_iters.metadata.stan_vars_dims)
         )
-        self.assertTrue('y_rep' in vars_iters)
+        self.assertIn('y_rep', vars_iters)
         self.assertEqual(vars_iters['y_rep'].shape, (8, 5, 4, 3))
-        self.assertTrue('beta' in vars_iters)
+        self.assertIn('beta', vars_iters)
         self.assertEqual(vars_iters['beta'].shape, (8, 2))
-        self.assertTrue('frac_60' in vars_iters)
+        self.assertIn('frac_60', vars_iters)
         self.assertEqual(vars_iters['frac_60'].shape, (8,))
 
 
@@ -580,7 +580,7 @@ class OptimizeTest(unittest.TestCase):
                 show_console=True,
             )
         console = sys_stdout.getvalue()
-        self.assertTrue('Chain [1] method = optimize' in console)
+        self.assertIn('Chain [1] method = optimize', console)
 
     def test_exe_only(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
@@ -609,6 +609,24 @@ class OptimizeTest(unittest.TestCase):
         # make sure the name 'imag' isn't magic
         self.assertEqual(fit.stan_variable('imag').shape, (2,))
 
+    def test_attrs(self):
+        stan = os.path.join(DATAFILES_PATH, 'named_output.stan')
+        model = CmdStanModel(stan_file=stan)
+        jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+        fit = model.optimize(data=jdata)
+
+        self.assertEqual(fit.a, 4.5)
+        self.assertEqual(fit.b.shape, (3,))
+        self.assertIsInstance(fit.theta, float)
+
+        self.assertEqual(fit.stan_variable('thin'), 3.5)
+
+        self.assertIsInstance(fit.optimized_params_np, np.ndarray)
+        self.assertEqual(fit.stan_variable('optimized_params_np'), 0)
+
+        with self.assertRaisesRegex(AttributeError, 'Unknown variable name:'):
+            dummy = fit.c
+
 
 if __name__ == '__main__':
     unittest.main()
