@@ -14,7 +14,7 @@ from test import CustomTestCase
 from time import time
 
 import numpy as np
-from testfixtures import LogCapture
+from testfixtures import LogCapture, StringComparison
 
 try:
     import ujson as json
@@ -62,12 +62,11 @@ class SampleTest(unittest.TestCase):
             iter_sampling=100,
             show_progress=False,
         )
-        self.assertIn('CmdStanMCMC: model=bernoulli', bern_fit.__repr__())
-        self.assertIn('method=sample', bern_fit.__repr__())
+        self.assertIn('CmdStanMCMC: model=bernoulli', repr(bern_fit))
+        self.assertIn('method=sample', repr(bern_fit))
 
         self.assertEqual(bern_fit.runset._args.method, Method.SAMPLE)
 
-        print(bern_fit.runset)
         for i in range(bern_fit.runset.chains):
             csv_file = bern_fit.runset.csv_files[i]
             stdout_file = bern_fit.runset.stdout_files[i]
@@ -99,8 +98,8 @@ class SampleTest(unittest.TestCase):
             metric='dense_e',
             show_progress=False,
         )
-        self.assertIn('CmdStanMCMC: model=bernoulli', bern_fit.__repr__())
-        self.assertIn('method=sample', bern_fit.__repr__())
+        self.assertIn('CmdStanMCMC: model=bernoulli', repr(bern_fit))
+        self.assertIn('method=sample', repr(bern_fit))
 
         self.assertEqual(bern_fit.runset._args.method, Method.SAMPLE)
 
@@ -221,7 +220,7 @@ class SampleTest(unittest.TestCase):
             inits=1.1,
             show_progress=False,
         )
-        self.assertIn('init=1.1', bern_fit.runset.__repr__())
+        self.assertIn('init=1.1', repr(bern_fit.runset))
 
         bern_fit = bern_model.sample(
             data=jdata,
@@ -233,7 +232,7 @@ class SampleTest(unittest.TestCase):
             inits=1,
             show_progress=False,
         )
-        self.assertIn('init=1', bern_fit.runset.__repr__())
+        self.assertIn('init=1', repr(bern_fit.runset))
 
         # Save init to json
         inits_path1 = os.path.join(_TMPDIR, 'inits_test_1.json')
@@ -255,7 +254,7 @@ class SampleTest(unittest.TestCase):
         )
         self.assertIn(
             'init={}'.format(inits_path1.replace('\\', '\\\\')),
-            bern_fit.runset.__repr__(),
+            repr(bern_fit.runset),
         )
 
         bern_fit = bern_model.sample(
@@ -270,7 +269,7 @@ class SampleTest(unittest.TestCase):
         )
         self.assertIn(
             'init={}'.format(inits_path1.replace('\\', '\\\\')),
-            bern_fit.runset.__repr__(),
+            repr(bern_fit.runset),
         )
 
         with self.assertRaises(ValueError):
@@ -317,7 +316,6 @@ class SampleTest(unittest.TestCase):
                 bern_model.sample(data=jdata, chains=1, output_dir=path)
             os.rmdir(dirname1)
 
-    # pylint: disable=no-self-use
     def test_multi_proc_1(self):
         logistic_stan = os.path.join(DATAFILES_PATH, 'logistic.stan')
         logistic_model = CmdStanModel(stan_file=logistic_stan)
@@ -338,7 +336,6 @@ class SampleTest(unittest.TestCase):
             ('cmdstanpy', 'INFO', 'Chain [2] start processing'),
         )
 
-    # pylint: disable=no-self-use
     def test_multi_proc_2(self):
         logistic_stan = os.path.join(DATAFILES_PATH, 'logistic.stan')
         logistic_model = CmdStanModel(stan_file=logistic_stan)
@@ -441,8 +438,8 @@ class SampleTest(unittest.TestCase):
                 cpp_options={'STAN_THREADS': 'TRUE'},
             )
             info_dict = logistic_model.exe_info()
-            self.assertTrue(info_dict is not None)
-            self.assertTrue('STAN_THREADS' in info_dict)
+            self.assertIsNotNone(info_dict)
+            self.assertIn('STAN_THREADS', info_dict)
             self.assertEqual(info_dict['STAN_THREADS'], 'true')
 
             logistic_data = os.path.join(DATAFILES_PATH, 'logistic.data.R')
@@ -489,6 +486,8 @@ class SampleTest(unittest.TestCase):
         self.assertEqual(datagen_fit.metric_type, None)
         self.assertEqual(datagen_fit.metric, None)
         self.assertEqual(datagen_fit.step_size, None)
+        self.assertEqual(datagen_fit.divergences, None)
+        self.assertEqual(datagen_fit.max_treedepths, None)
 
         for i in range(datagen_fit.runset.chains):
             csv_file = datagen_fit.runset.csv_files[i]
@@ -615,7 +614,7 @@ class SampleTest(unittest.TestCase):
 
     def test_bernoulli_path_with_space(self):
         self.test_bernoulli_good(
-            'path with space/' 'bernoulli_path_with_space.stan'
+            'path with space/' + 'bernoulli_path_with_space.stan'
         )
 
     def test_index_bounds_error(self):
@@ -642,8 +641,8 @@ class SampleTest(unittest.TestCase):
                 show_console=True,
             )
         console = sys_stdout.getvalue()
-        self.assertTrue('Chain [1] method = sample' in console)
-        self.assertTrue('Chain [2] method = sample' in console)
+        self.assertIn('Chain [1] method = sample', console)
+        self.assertIn('Chain [2] method = sample', console)
 
     def test_show_progress(self, stanfile='bernoulli.stan'):
         stan = os.path.join(DATAFILES_PATH, stanfile)
@@ -660,9 +659,9 @@ class SampleTest(unittest.TestCase):
                 show_progress=True,
             )
         console = sys_stderr.getvalue()
-        self.assertTrue('chain 1' in console)
-        self.assertTrue('chain 2' in console)
-        self.assertTrue('Sampling completed' in console)
+        self.assertIn('chain 1', console)
+        self.assertIn('chain 2', console)
+        self.assertIn('Sampling completed', console)
 
         sys_stderr = io.StringIO()  # tqdm prints to stderr
         with contextlib.redirect_stderr(sys_stderr):
@@ -674,9 +673,9 @@ class SampleTest(unittest.TestCase):
                 show_progress=True,
             )
         console = sys_stderr.getvalue()
-        self.assertTrue('chain 6' in console)
-        self.assertTrue('chain 7' in console)
-        self.assertTrue('Sampling completed' in console)
+        self.assertIn('chain 6', console)
+        self.assertIn('chain 7', console)
+        self.assertIn('Sampling completed', console)
         sys_stderr = io.StringIO()  # tqdm prints to stderr
 
         with contextlib.redirect_stderr(sys_stderr):
@@ -690,9 +689,9 @@ class SampleTest(unittest.TestCase):
                 show_progress=True,
             )
         console = sys_stderr.getvalue()
-        self.assertTrue('chain 6' in console)
-        self.assertTrue('chain 7' in console)
-        self.assertTrue('Sampling completed' in console)
+        self.assertIn('chain 6', console)
+        self.assertIn('chain 7', console)
+        self.assertIn('Sampling completed', console)
 
 
 class CmdStanMCMCTest(CustomTestCase):
@@ -922,7 +921,6 @@ class CmdStanMCMCTest(CustomTestCase):
         fixed_param_sample = from_csv(path=csv_path)
         self.assertEqual(fixed_param_sample.draws_pd().shape, (100, 85))
 
-    # pylint: disable=no-self-use
     def test_custom_metric(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
@@ -1241,6 +1239,18 @@ class CmdStanMCMCTest(CustomTestCase):
         ):
             CmdStanMCMC(runset)
 
+    def test_sample_sporadic_exception(self):
+        stan = os.path.join(DATAFILES_PATH, 'linear_regression.stan')
+        jdata = os.path.join(DATAFILES_PATH, 'linear_regression.data.json')
+        linear_model = CmdStanModel(stan_file=stan)
+        # will produce a failure due to calling normal_lpdf with 0 for scale
+        # but then continue sampling normally
+        with LogCapture() as log:
+            linear_model.sample(data=jdata, inits=0)
+        log.check_present(
+            ('cmdstanpy', 'WARNING', StringComparison(r"Non-fatal error.*"))
+        )
+
     def test_save_warmup(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
@@ -1388,7 +1398,7 @@ class CmdStanMCMCTest(CustomTestCase):
             data=jdata, chains=2, seed=12345, iter_warmup=100, iter_sampling=100
         )
         self.assertEqual(1, len(bern_fit.metadata.stan_vars_dims))
-        self.assertTrue('theta' in bern_fit.metadata.stan_vars_dims)
+        self.assertIn('theta', bern_fit.metadata.stan_vars_dims)
         self.assertEqual(bern_fit.metadata.stan_vars_dims['theta'], ())
         self.assertEqual(bern_fit.stan_variable(var='theta').shape, (200,))
         with self.assertRaises(ValueError):
@@ -1401,13 +1411,13 @@ class CmdStanMCMCTest(CustomTestCase):
         fit = from_csv(path=csvfiles_path)
         self.assertEqual(20, fit.num_draws_sampling)
         self.assertEqual(8, len(fit.metadata.stan_vars_dims))
-        self.assertTrue('z' in fit.metadata.stan_vars_dims)
+        self.assertIn('z', fit.metadata.stan_vars_dims)
         self.assertEqual(fit.metadata.stan_vars_dims['z'], (20, 2))
         vars = fit.stan_variables()
         self.assertEqual(len(vars), len(fit.metadata.stan_vars_dims))
-        self.assertTrue('z' in vars)
+        self.assertIn('z', vars)
         self.assertEqual(vars['z'].shape, (20, 20, 2))
-        self.assertTrue('theta' in vars)
+        self.assertIn('theta', vars)
         self.assertEqual(vars['theta'].shape, (20, 4))
 
     def test_variables_3d(self):
@@ -1416,7 +1426,7 @@ class CmdStanMCMCTest(CustomTestCase):
         fit = from_csv(path=csvfiles_path)
         self.assertEqual(20, fit.num_draws_sampling)
         self.assertEqual(3, len(fit.metadata.stan_vars_dims))
-        self.assertTrue('y_rep' in fit.metadata.stan_vars_dims)
+        self.assertIn('y_rep', fit.metadata.stan_vars_dims)
         self.assertEqual(fit.metadata.stan_vars_dims['y_rep'], (5, 4, 3))
         var_y_rep = fit.stan_variable(var='y_rep')
         self.assertEqual(var_y_rep.shape, (20, 5, 4, 3))
@@ -1426,11 +1436,11 @@ class CmdStanMCMCTest(CustomTestCase):
         self.assertEqual(var_frac_60.shape, (20,))
         vars = fit.stan_variables()
         self.assertEqual(len(vars), len(fit.metadata.stan_vars_dims))
-        self.assertTrue('y_rep' in vars)
+        self.assertIn('y_rep', vars)
         self.assertEqual(vars['y_rep'].shape, (20, 5, 4, 3))
-        self.assertTrue('beta' in vars)
+        self.assertIn('beta', vars)
         self.assertEqual(vars['beta'].shape, (20, 2))
-        self.assertTrue('frac_60' in vars)
+        self.assertIn('frac_60', vars)
         self.assertEqual(vars['frac_60'].shape, (20,))
 
     def test_variables_issue_361(self):
@@ -1592,18 +1602,16 @@ class CmdStanMCMCTest(CustomTestCase):
         fit = CmdStanMCMC(runset)
         meta = fit.metadata
         self.assertEqual(meta.cmdstan_config['model'], 'logistic_model')
-        col_names = tuple(
-            [
-                'lp__',
-                'accept_stat__',
-                'stepsize__',
-                'treedepth__',
-                'n_leapfrog__',
-                'divergent__',
-                'energy__',
-                'beta[1]',
-                'beta[2]',
-            ]
+        col_names = (
+            'lp__',
+            'accept_stat__',
+            'stepsize__',
+            'treedepth__',
+            'n_leapfrog__',
+            'divergent__',
+            'energy__',
+            'beta[1]',
+            'beta[2]',
         )
 
         self.assertEqual(fit.chains, 4)
@@ -1619,14 +1627,14 @@ class CmdStanMCMCTest(CustomTestCase):
         self.assertEqual(fit.metadata.cmdstan_config['metric'], 'diag_e')
         self.assertAlmostEqual(fit.metadata.cmdstan_config['delta'], 0.80)
 
-        self.assertTrue('n_leapfrog__' in fit.metadata.method_vars_cols)
-        self.assertTrue('energy__' in fit.metadata.method_vars_cols)
-        self.assertTrue('beta' not in fit.metadata.method_vars_cols)
-        self.assertTrue('energy__' not in fit.metadata.stan_vars_dims)
-        self.assertTrue('beta' in fit.metadata.stan_vars_dims)
-        self.assertTrue('beta' in fit.metadata.stan_vars_cols)
-        self.assertEqual(fit.metadata.stan_vars_dims['beta'], tuple([2]))
-        self.assertEqual(fit.metadata.stan_vars_cols['beta'], tuple([7, 8]))
+        self.assertIn('n_leapfrog__', fit.metadata.method_vars_cols)
+        self.assertIn('energy__', fit.metadata.method_vars_cols)
+        self.assertNotIn('beta', fit.metadata.method_vars_cols)
+        self.assertNotIn('energy__', fit.metadata.stan_vars_dims)
+        self.assertIn('beta', fit.metadata.stan_vars_dims)
+        self.assertIn('beta', fit.metadata.stan_vars_cols)
+        self.assertEqual(fit.metadata.stan_vars_dims['beta'], (2,))
+        self.assertEqual(fit.metadata.stan_vars_cols['beta'], (7, 8))
 
     def test_save_latent_dynamics(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
@@ -1762,6 +1770,20 @@ class CmdStanMCMCTest(CustomTestCase):
                 self.assertEqual(int(z_as_ndarray[0, i, j]), i + 1)
                 self.assertEqual(int(z_as_xr.z.data[0, 0, i, j]), i + 1)
 
+    def test_overlapping_names(self):
+        stan = os.path.join(DATAFILES_PATH, 'normal-rng.stan')
+
+        mod = CmdStanModel(stan_file=stan)
+        # %Y to force same names
+        fits = [
+            mod.sample(data={}, time_fmt="%Y", iter_sampling=1, iter_warmup=1)
+            for i in range(10)
+        ]
+
+        self.assertEqual(
+            len(np.unique([fit.stan_variables()["x"][0] for fit in fits])), 10
+        )
+
     def test_complex_output(self):
         stan = os.path.join(DATAFILES_PATH, 'complex_var.stan')
         model = CmdStanModel(stan_file=stan)
@@ -1778,6 +1800,87 @@ class CmdStanMCMCTest(CustomTestCase):
             fit.draws_xr().z.isel(chain=0, draw=1).data[()], 3 + 4j
         )
 
+    def test_attrs(self):
+        stan = os.path.join(DATAFILES_PATH, 'named_output.stan')
+        model = CmdStanModel(stan_file=stan)
+        jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+        fit = model.sample(chains=1, iter_sampling=10, data=jdata)
+
+        self.assertEqual(fit.a[0], 4.5)
+        self.assertEqual(fit.b.shape, (10, 3))
+        self.assertEqual(fit.theta.shape, (10,))
+
+        self.assertEqual(fit.thin, 1)
+        self.assertEqual(fit.stan_variable('thin')[0], 3.5)
+
+        fit.draws()
+        self.assertEqual(fit.stan_variable('draws')[0], 0)
+
+        with self.assertRaisesRegex(AttributeError, 'Unknown variable name:'):
+            dummy = fit.c
+
+    def test_diagnostics(self):
+        # centered 8 schools hits funnel
+        stan = os.path.join(DATAFILES_PATH, 'eight_schools.stan')
+        model = CmdStanModel(stan_file=stan)
+        rdata = os.path.join(DATAFILES_PATH, 'eight_schools.data.R')
+        with LogCapture(level=logging.WARNING) as log:
+            logging.getLogger()
+            fit = model.sample(
+                data=rdata,
+                seed=55157,
+            )
+            log.check_present(
+                (
+                    'cmdstanpy',
+                    'WARNING',
+                    StringComparison(
+                        r'(?s).*Some chains may have failed to converge.*'
+                    ),
+                )
+            )
+            self.assertFalse(np.all(fit.divergences == 0))
+
+        with LogCapture(level=logging.WARNING) as log:
+            logging.getLogger()
+            fit = model.sample(
+                data=rdata,
+                seed=40508,
+                max_treedepth=3,
+            )
+            log.check_present(
+                (
+                    'cmdstanpy',
+                    'WARNING',
+                    StringComparison(r'(?s).*max treedepth*'),
+                )
+            )
+            self.assertFalse(np.all(fit.max_treedepths == 0))
+
+        stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+        model = CmdStanModel(stan_file=stan)
+        jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+        fit = model.sample(
+            data=jdata,
+            iter_warmup=200,
+            iter_sampling=100,
+        )
+        self.assertTrue(np.all(fit.divergences == 0))
+        self.assertTrue(np.all(fit.max_treedepths == 0))
+
+        # fixed_param returns None
+        stan = os.path.join(DATAFILES_PATH, 'container_vars.stan')
+        container_vars_model = CmdStanModel(stan_file=stan)
+        fit = container_vars_model.sample(
+            chains=1,
+            iter_sampling=4,
+            fixed_param=True,
+            show_progress=False,
+            show_console=False,
+        )
+        self.assertEqual(fit.max_treedepths, None)
+        self.assertEqual(fit.divergences, None)
+
 
 if __name__ == '__main__':
     unittest.main()
